#include <satplc.h>
#include <satkbd.h>
#include <satmenu.h>
#include <satmio.h>
#include <satscr.h>
#include <satser.h>
#include <modbus.h>
#include <satpid.h>
#include <satmem.h>

// параметры
struct MY_MODBUS_DATA /*__attribute__((packed))*/
{
    unsigned short int remoteOnOffPump; // 0
    unsigned short int unused;          // 1
    int remoteControl;                  // 2-3
    int resetMotoHours;                 // 4-5
    int resetAlarm;                     // 6-7
    int resetTimer;                     // 8-9
    float autoWarmOn;                   // 10-11
    float autoWarmOff;                  // 12-13
    int DDRepair;                       // 14-15
    float startFlowmeter;               // 16-17
    int warmOn;                         // 18-19
    int DDLimit;                        // 20-21
    int resetAlarmNumber;               // 22-23
    int minFrequency;                   // 24-25
    int setPoint;                       // 26-27
    int fixedFrequency;                 // 28-29
    int pumpRepair;                     // 30-31
    int fixedFrequencyOn;               // 32-33
    float oneImpulse;                   // 34-35
    int sleepInFrequency;               // 36-37
    int sleepTime;                      // 38-39
    int lowPressure;                    // 40-41
    float Kp;                           // 42-43
    float Ki;                           // 44-45
    float Kd;                           // 46-47
    float dt;                           // 48-49

    unsigned short int KZ_DD;         // 50
    unsigned short int Break_DD;      // 51
    unsigned short int dryLvl;        // 52
    unsigned short int wetLvl;        // 53
    unsigned short int globalSeconds; // 54
    unsigned short int globalMinutes; // 55
    unsigned short int globalHours;   // 56
    unsigned short int pressureDD;    // 57
    unsigned short int outsUZ[6];     // 63
    float instant;                    // 64-65
    float globalPump;                 // 66-67
    float temperature;                // 68-69
};
struct MY_MODBUS_DATA ModbusData;

unsigned short int GeneratorDD(unsigned short int max);
void mainMenu(PIDController pid);
void checkWarm(void);
void Start(void);
void regulatePump(void);
void CheckDD(void);
void checkUZ(void);
void globalPumpTimer(void);
void checkFix(void);
void calcPump(void);
void MainLoop(PIDController pid);
void checkAlarmReset(void);
void alarmPage(PIDController pid);
void UZPage(PIDController pid);
void initializeModbusDefault(void);
void initializeParams(void);
void initializeCoils(void);
void getFromBackupRam(void);
bool isPrevious(void);
void writeToRam(void);

unsigned char DD_OFF = 0;
unsigned char Fault_DD = 0;
unsigned char Reset_Pump = 0;
unsigned char Alarm_UZ = 0;
unsigned char OnOffPump = 0;
unsigned char stringMB = 1;
unsigned char countTry = 0;    // счётчик попыток автосброса аварий
unsigned int prevSec = 0;      // предыдущее значение секунд для расходомера
unsigned int prevAlarmSec = 0; // предыдущее значение секунд для автосброса аварий
unsigned int prevSecAlarmUZ = 0;
unsigned int prevSleepSec = 0; // предыдущее значение секунд для входа в сон
unsigned int t1 = 0;

void initializeModbusDefault(void)
{
    ModbusData.remoteOnOffPump = 0;     // 1
    ModbusData.remoteControl = 0;       // 3 + 2
    ModbusData.resetMotoHours = 0;      // 5
    ModbusData.resetAlarm = 0;          // 7
    ModbusData.resetTimer = 0;          // 9
    ModbusData.autoWarmOn = 5.0f;       // 11 + 4
    ModbusData.autoWarmOff = 10.0f;     // 13 + 6
    ModbusData.DDRepair = 0;            // 15 + 8
    ModbusData.startFlowmeter = 200.0f; // 17 + 10
    ModbusData.warmOn = 0;              // 19 + 12
    ModbusData.DDLimit = 400;           // 21 + 14
    ModbusData.resetAlarmNumber = 3;    // 23 + 16
    ModbusData.minFrequency = 20;       // 25 + 18
    ModbusData.setPoint = 200;          // 27 + 20
    ModbusData.fixedFrequency = 45;     // 29 + 22
    ModbusData.pumpRepair = 0;          // 31 + 24
    ModbusData.fixedFrequencyOn = 0;    // 33 + 26
    ModbusData.oneImpulse = 0.1f;       // 35 + 28
    ModbusData.sleepInFrequency = 30;   // 37 + 30
    ModbusData.sleepTime = 20;          // 39 + 32
    ModbusData.lowPressure = 130;       // 41 + 34
    ModbusData.Kp = 0.02f;              // 43 + 36
    ModbusData.Ki = 0.3f;               // 45 + 38
    ModbusData.Kd = 0.0f;               // 47 + 40
    ModbusData.dt = 0.0f;               // 49 + 42
}

struct RamData
{
    int remoteControl;
    float autoWarmOn;
    float autoWarmOff;
    int DDRepair;
    float startFlowmeter;
    int warmOn;
    int DDLimit;
    int resetAlarmNumber;
    int minFrequency;
    int setPoint;
    int fixedFrequency;
    int pumpRepair;
    int fixedFrequencyOn;
    float oneImpulse;
    int sleepInFrequency;
    int sleepTime;
    int lowPressure;
    float Kp;
    float Ki;
    float Kd;
    float dt;
};

struct RamData ram;

void writeToRam(void)
{
    ram.remoteControl = ModbusData.remoteControl;
    ram.autoWarmOn = ModbusData.autoWarmOn;
    ram.autoWarmOff = ModbusData.autoWarmOff;
    ram.DDRepair = ModbusData.DDRepair;
    ram.startFlowmeter = ModbusData.startFlowmeter;
    ram.warmOn = ModbusData.warmOn;
    ram.DDLimit = ModbusData.DDLimit;
    ram.resetAlarmNumber = ModbusData.resetAlarmNumber;
    ram.minFrequency = ModbusData.minFrequency;
    ram.setPoint = ModbusData.setPoint;
    ram.fixedFrequency = ModbusData.fixedFrequency;
    ram.pumpRepair = ModbusData.pumpRepair;
    ram.fixedFrequencyOn = ModbusData.fixedFrequencyOn;
    ram.oneImpulse = ModbusData.oneImpulse;
    ram.sleepInFrequency = ModbusData.sleepInFrequency;
    ram.sleepTime = ModbusData.sleepTime;
    ram.lowPressure = ModbusData.lowPressure;
    ram.Kp = ModbusData.Kp;
    ram.Ki = ModbusData.Ki;
    ram.Kd = ModbusData.Kd;
    ram.dt = ModbusData.dt;
    BackupRamWrite(0, (unsigned char *)&ram, sizeof(ram));
}

bool isPrevious(void)
{
    for (unsigned char i = 0; i <= 41; i++)
    {
        if (BackupRamRead8(i) != 0)
            return true;
    }
    return false;
}

void getFromBackupRam(void)
{
    BackupRamRead(0, (unsigned char *)&ram, sizeof(ram));
    ModbusData.remoteControl = ram.remoteControl;
    ModbusData.autoWarmOn = ram.autoWarmOn;
    ModbusData.autoWarmOff = ram.autoWarmOff;
    ModbusData.DDRepair = ram.DDRepair;
    ModbusData.startFlowmeter = ram.startFlowmeter;
    ModbusData.warmOn = ram.warmOn;
    ModbusData.DDLimit = ram.DDLimit;
    ModbusData.resetAlarmNumber = ram.resetAlarmNumber;
    ModbusData.minFrequency = ram.minFrequency;
    ModbusData.setPoint = ram.setPoint;
    ModbusData.fixedFrequency = ram.fixedFrequency;
    ModbusData.pumpRepair = ram.pumpRepair;
    ModbusData.fixedFrequencyOn = ram.fixedFrequencyOn;
    ModbusData.oneImpulse = ram.oneImpulse;
    ModbusData.sleepInFrequency = ram.sleepInFrequency;
    ModbusData.sleepTime = ram.sleepTime;
    ModbusData.lowPressure = ram.lowPressure;
    ModbusData.Kp = ram.Kp;
    ModbusData.Ki = ram.Ki;
    ModbusData.Kd = ram.Kd;
    ModbusData.dt = ram.dt;
}

void initializeCoils(void)
{
    // DI1-DI9
    SetSigName(0, 0, "Режим Авто");
    SetSigName(0, 1, "Работа ШУ от сети");
    SetSigName(0, 2, "Низкий заряд АКБ");
    SetSigName(0, 3, "Работа ПЧ");
    SetSigName(0, 4, "Контроль доступа");
    SetSigName(0, 5, "Расходомер");
    SetSigName(0, 6, "Резерв");
    SetSigName(0, 7, "Резерв");
    // DO1-DO3 и DO6
    SetSigName(1, 0, "Работа насоса 1");
    SetSigName(1, 1, "Авария насоса 1");
    SetSigName(1, 2, "Пуск ПЧ");
    SetSigName(1, 5, "Обогреватель");
    // AI1
    SetSigName(2, 0, "ДД скважины");
    // AO1
    SetSigName(3, 0, "Задание частоты");
    // T1-T3
    SetSigName(4, 0, "Температура");
    SetSigName(4, 1, "Сухой ход");
    SetSigName(4, 2, "Затопление");
}

void initializeParams(void)
{
    AddSystemMenuBoolItem("Разреш. удал. пуск ПЧ", (int *)&ModbusData.remoteControl);
    AddSystemMenuBoolItem("Сброс аварий", (int *)&ModbusData.resetAlarm);
    AddSystemMenuBoolItem("Сброс моточасов скважин", (int *)&ModbusData.resetMotoHours);
    AddSystemMenuBoolItem("Сброс счётчика", (int *)&ModbusData.resetTimer);
    AddSystemMenuFloatItem("Температура ВКЛ обогрева", (float *)&ModbusData.autoWarmOn, -50.0f, 70.0f, 1);
    AddSystemMenuFloatItem("Температура ВЫКЛ обогрева", (float *)&ModbusData.autoWarmOff, -50.0f, 70.0f, 1);
    AddSystemMenuBoolItem("Вывод ДД скв. в ремонт", (int *)&ModbusData.DDRepair);
    AddSystemMenuFloatItem("Начальные данные расход.", (float *)&ModbusData.startFlowmeter, 0.0f, 1000.0f, 1);
    AddSystemMenuBoolItem("Ручной ВКЛ обогревателя", (int *)&ModbusData.warmOn);
    AddSystemMenuIntItem("Верхний предел ДД", (int *)&ModbusData.DDLimit, 0, 1000);
    AddSystemMenuIntItem("Кол-во попыток сброса", (int *)&ModbusData.resetAlarmNumber, 0, 5);
    AddSystemMenuIntItem("Мин. допустим. частота", (int *)&ModbusData.minFrequency, 0, 50);
    AddSystemMenuIntItem("Уставка давления, кПа", (int *)&ModbusData.setPoint, 0, 1000);
    AddSystemMenuIntItem("Фикс. частота ПЧ", (int *)&ModbusData.fixedFrequency, 0, 50);
    AddSystemMenuBoolItem("Вывод скв. в ремонт", (int *)&ModbusData.pumpRepair);
    AddSystemMenuBoolItem("Работа на фикс. частоте", (int *)&ModbusData.fixedFrequencyOn);
    AddSystemMenuFloatItem("1 импульс, м3", (float *)&ModbusData.oneImpulse, 0.0f, 10.0f, 2);
    AddSystemMenuIntItem("Частота входа в сон", (int *)&ModbusData.sleepInFrequency, 0, 50);
    AddSystemMenuIntItem("Задержка входа в сон", (int *)&ModbusData.sleepTime, 1, 40);
    AddSystemMenuIntItem("Низкое давление, кПа", (int *)&ModbusData.lowPressure, 0, 1000);
    AddSystemMenuFloatItem("Коэф. P", (float *)&ModbusData.Kp, 0.0f, 100.0f, 2);
    AddSystemMenuFloatItem("Коэф. I", (float *)&ModbusData.Ki, 0.0f, 100.0f, 2);
    AddSystemMenuFloatItem("Коэф. D", (float *)&ModbusData.Kd, 0.0f, 100.0f, 2);
    AddSystemMenuFloatItem("Коэф. dt", (float *)&ModbusData.dt, 0.0f, 5.0f, 1);
}

void main(void)
{
    SetDO(DO1, 0);
    SetDO(DO2, 0);
    SetDO(DO3, 0);
    SetDO(DO6, 0);
    if (isPrevious())
        getFromBackupRam();
    else
        initializeModbusDefault();
    SetAO(0, ModbusData.minFrequency * 0.2f);
    StartModbusTCPServer((unsigned short int *)&ModbusData.remoteOnOffPump, 70, NULL, 0);
    ModbusData.globalPump += ModbusData.startFlowmeter;
    initializeCoils();
    initializeParams();
    SetEnableKeyboard(1);
    PIDController pid = {ModbusData.Kp, ModbusData.Ki, ModbusData.Kd, 0.02f, -10.0f, 10.0f, -5.0f, 5.0f, ModbusData.dt};
    PIDController_Init(&pid);
    mainMenu(pid);
}
// все ок

void MainLoop(PIDController pid)
{
    checkWarm();
    Start();
    CheckDD();
    regulatePump();
    checkUZ();
    calcPump();
    checkAlarmReset();
    pid.Kp = ModbusData.Kp;
    pid.Ki = ModbusData.Ki;
    pid.Kd = ModbusData.Kd;
    pid.T = ModbusData.dt;
    if (DD_OFF || ModbusData.fixedFrequencyOn)
    {
        SetAO(AO1, ModbusData.fixedFrequency * 0.2f);
        if (GetAO(AO1) < ModbusData.minFrequency * 0.2f)
            SetAO(AO1, ModbusData.minFrequency * 0.2f);
    }
    else
    {
        PIDController_Update(&pid, ModbusData.setPoint, ModbusData.pressureDD);
        if (GetAO(AO1) + pid.out < ModbusData.minFrequency * 0.2f)
        {
            SetAO(AO1, ModbusData.minFrequency * 0.2f);
        }
        else
        {
            if (GetAO(AO1) + pid.out > 10.0f)
                SetAO(AO1, 10.0f);
            else
                SetAO(AO1, GetAO(AO1) + pid.out);
        }
    }
    writeToRam();
}
// все ок

void mainMenu(PIDController pid)
{
    BigFont = 0;
    while (1)
    {
        if (GetTickCount() - t1 > 1000)
        {
            t1 = GetTickCount();
            globalPumpTimer();
        }
        MainLoop(pid);
        BrushColor = COLOR_BLACK;
        FontColor = COLOR_WHITE;
        ClrScr();
        puts("Скважина: ");
        if (Alarm_UZ)
        {
            BrushColor = COLOR_RED;
            FontColor = COLOR_YELLOW;
            puts("АВАРИЯ\n");
        }
        else
        {
            if (GetDI(DI4) && GetDO(DO3))
            {
                BrushColor = COLOR_GREEN;
                FontColor = COLOR_MAGENTA;
                puts("РАБОТА\n");
            }
            else
            {
                BrushColor = COLOR_YELLOW;
                FontColor = COLOR_BLACK;
                printf("СОН\n");
            }
        }
        BrushColor = COLOR_BLACK;
        FontColor = COLOR_WHITE;
        puts("Режим: ");
        if (ModbusData.pumpRepair)
        {
            BrushColor = COLOR_GRAY;
            FontColor = COLOR_BLACK;
            puts("РЕМОНТ\n");
        }
        else
        {
            if (GetDI(DI1))
            {
                BrushColor = COLOR_GREEN;
                FontColor = COLOR_MAGENTA;
                puts("АВТО\n");
            }
            else
            {
                BrushColor = COLOR_CYAN;
                FontColor = COLOR_ORANGE;
                puts("РУЧНОЙ\n");
            }
        }
        BrushColor = COLOR_BLACK;
        FontColor = COLOR_WHITE;
        if (DD_OFF || ModbusData.fixedFrequencyOn)
            printf("Фикс. частота %d Гц\n", ModbusData.fixedFrequency);
        else
            printf("Задан. частота %.2f Гц\n", GetAO(AO1) * 5);
        if (ModbusData.DDRepair)
        {
            BrushColor = COLOR_GRAY;
            FontColor = COLOR_BLACK;
            puts("Ремонт ДД в скв.\n");
        }
        else
        {
            if (GetAI(AI1) < 2.0f)
            {
                BrushColor = COLOR_RED;
                FontColor = COLOR_YELLOW;
                puts("Обрыв ДД в скв.\n");
            }
            else
            {
                if (GetAI(AI1) > 25.0f)
                {
                    BrushColor = COLOR_RED;
                    FontColor = COLOR_YELLOW;
                    puts("К.З. ДД в скв.\n");
                }
                else
                    printf("Давление в скв. %d КПа\n", ModbusData.pressureDD);
            }
        }
        BrushColor = COLOR_GREEN;
        FontColor = COLOR_BLACK;
        if (GetT(TEMP1) <= ModbusData.autoWarmOn)
        {
            BrushColor = COLOR_CYAN;
            FontColor = COLOR_BLUE;
        }
        if (GetT(TEMP1) >= ModbusData.autoWarmOff)
        {
            BrushColor = COLOR_RED;
            FontColor = COLOR_YELLOW;
        }
        printf("Температура %.1f C\n", GetT(TEMP1));
        BrushColor = COLOR_BLACK;
        FontColor = COLOR_WHITE;
        printf("Нараб. скв: %05dч %02dм %02dс\n", ModbusData.globalHours, ModbusData.globalMinutes, ModbusData.globalSeconds);
        printf("Накопленный расход: %.2f\n", ModbusData.globalPump);
        printf("Мгновенный расход: %.2f\n", ModbusData.instant);
        ShowScr();
        msleep(300);
        unsigned char key = GetKey();
        switch (key)
        {
        case KEY_RIGHT:
            SystemMenu();
            break;
        case KEY_DOWN:
            alarmPage(pid);
            break;
        case KEY_UP:
            UZPage(pid);
            break;
        }
    }
}
// все ок

void alarmPage(PIDController pid)
{
    BrushColor = COLOR_WHITE;
    FontColor = COLOR_BLACK;
    while (1)
    {
        MainLoop(pid);
        ClrScr();
        puts("СПИСОК АВАРИЙ\n");
        if (!GetDI(DI2))
            puts("Работа ШУ от АКБ\n");
        if (GetDI(DI3))
            puts("Низкий заряд АКБ\n");
        if (ModbusData.wetLvl)
            puts("Затопление\n");
        if (ModbusData.dryLvl)
            puts("Сухой ход\n");
        if (Alarm_UZ)
            puts("Авария запуска скв.\n");
        if (Fault_DD)
            puts("Неисправен датчик скв.\n");
        ShowScr();
        msleep(300);
        unsigned char key = GetKey();
        switch (key)
        {
        case KEY_RIGHT:
            SystemMenu();
            break;
        case KEY_DOWN:
            UZPage(pid);
            break;
        case KEY_UP:
            mainMenu(pid);
            break;
        }
    }
}
// все ок

void UZPage(PIDController pid)
{
    BigFont = 0;
    BrushColor = COLOR_WHITE;
    FontColor = COLOR_BLACK;
    while (1)
    {
        MainLoop(pid);
        ClrScr();
        puts("ПЧ MODBUS\n");
        ModbusRTUTimeout = 1000;
        int handle = SerialOpen(1, B9600, PARNONE, 1);
        mdbs_result_t mdbs_result;
        mdbs_result = ModbusReadHoldingRegisters(handle, 16, 10099, 6, ModbusData.outsUZ);
        if (mdbs_result == MDBS_OK)
        {
            printf("Состояние: %d\n", ModbusData.outsUZ[0]);
            printf("Код ошибки: %d\n", ModbusData.outsUZ[1]);
            printf("Вых. частота: %d.%d Гц\n", ModbusData.outsUZ[2] / 10, ModbusData.outsUZ[2] % 10);
            printf("Вых. ток: %d.%02d А\n", ModbusData.outsUZ[3] / 100, ModbusData.outsUZ[3] % 100);
            printf("Вых. напряжение: %d В\n", ModbusData.outsUZ[4]);
            printf("Вых. мощность: %d.%02d кВт\n", ModbusData.outsUZ[5] / 100, ModbusData.outsUZ[5] % 100);
        }
        else
            printf("Нет подключения к ПЧ. Ошибка=%d\n", mdbs_result);
        SerialClose(&handle);
        ShowScr();
        msleep(300);
        unsigned char key = GetKey();
        switch (key)
        {
        case KEY_RIGHT:
            SystemMenu();
            break;
        case KEY_DOWN:
            mainMenu(pid);
            break;
        case KEY_UP:
            alarmPage(pid);
            break;
        }
    }
}
// все ок

unsigned short int GeneratorDD(unsigned short int max)
{
    if (GetAI(AI1) < 4.0f)
        return 0;
    if (GetAI(AI1) > 20.0f)
        return max;
    return ((GetAI(AI1) - 4) * max / 16);
}
// все ок

void checkWarm(void)
{
    ModbusData.temperature = GetT(TEMP1);
    if (ModbusData.temperature >= ModbusData.autoWarmOff && GetDO(DO6) && !ModbusData.warmOn)
        SetDO(DO6, 0);
    if ((ModbusData.temperature <= ModbusData.autoWarmOn || ModbusData.warmOn) && !GetDO(DO6))
        SetDO(DO6, 1);
}
// все ок

void Start(void)
{
    if (ModbusData.fixedFrequencyOn)
    {
        SetDO(DO3, GetDI(DI1) && !ModbusData.pumpRepair && !Alarm_UZ && !ModbusData.dryLvl);
    }
    else
    {
        if (ModbusData.remoteControl)
        {
            SetDO(DO3, GetDI(DI1) && !ModbusData.pumpRepair && !Alarm_UZ && ModbusData.remoteOnOffPump && !ModbusData.dryLvl);
        }
        else
            SetDO(DO3, GetDI(DI1) && !ModbusData.pumpRepair && !Alarm_UZ && OnOffPump && !ModbusData.dryLvl);
    }
}
// все ок

void CheckDD(void)
{
    DD_OFF = GetAI(AI1) > 25.0f || GetAI(AI1) < 2.0f || ModbusData.DDRepair;
    ModbusData.KZ_DD = GetAI(AI1) > 25.0f && !ModbusData.DDRepair;
    ModbusData.Break_DD = GetAI(AI1) < 2.0f && !ModbusData.DDRepair;
    Fault_DD = ModbusData.KZ_DD || ModbusData.Break_DD;
}
// все ок

void regulatePump(void)
{
    unsigned int now = GetTickCount();
    ModbusData.dryLvl = GetT(TEMP2) < 0.0f;
    ModbusData.wetLvl = GetT(TEMP3) > 0.0f;

    if (GetAO(AO1) > ModbusData.sleepInFrequency * 0.2f || !OnOffPump || ModbusData.pressureDD < ModbusData.lowPressure)
        prevSleepSec = now;

    unsigned int intervalSleep = now - prevSleepSec;

    if (intervalSleep >= ModbusData.sleepTime * 1000 && OnOffPump)
        OnOffPump = 0;
    if (ModbusData.pressureDD < ModbusData.lowPressure && !OnOffPump)
        OnOffPump = 1;
}
// все ок

void checkUZ(void)
{
    SetDO(DO1, GetDI(DI4) && GetDO(DO3));
    unsigned int now = GetTickCount();
    if (!GetDI(DI4) && GetDO(DO3))
    {
        unsigned int interval = now - prevSecAlarmUZ;
        if (!GetDI(DI4) && GetDO(DO3) && interval > 2000)
            Alarm_UZ = 1;
    }
    else
    {
        prevSecAlarmUZ = now;
    }
    if (Reset_Pump && Alarm_UZ)
    {
        ModbusRTUTimeout = 1000;
        int handle = SerialOpen(1, B9600, PARNONE, 1);
        mdbs_result_t mdbs_result;
        mdbs_result = ModbusWriteSingleRegister(handle, 16, 9999, 0x07);
        if (mdbs_result == MDBS_OK)
        {
            stringMB = 2;
        }
        else
        {
            stringMB = 3;
        }

        SerialClose(&handle);
        Alarm_UZ = 0;
        Reset_Pump = 0;
    }
    SetDO(DO2, Fault_DD || GetDI(DI3) || !GetDI(DI2) || Alarm_UZ || ModbusData.wetLvl || ModbusData.dryLvl);
}
// все ок

void checkAlarmReset(void)
{
    unsigned int now = GetTickCount();
    if (Alarm_UZ && (countTry < ModbusData.resetAlarmNumber) && (now - prevAlarmSec) > 10000)
    {
        Reset_Pump = 1;
        countTry++;
        prevAlarmSec = now;
    }
    if (ModbusData.resetAlarm)
    {
        Reset_Pump = 1;
        countTry = 0;
        ModbusData.resetAlarm = 0;
        prevAlarmSec = 0;
    }
}
// все ок

void calcPump(void)
{
    unsigned int now = GetTickCount();
    if (ModbusData.resetTimer)
    {
        prevSec = now;
        ModbusData.resetTimer = 0;
    }
    unsigned int interval = now - prevSec;
    // Если прошло больше 5 секунд
    if (interval >= 5000 && GetDI(DI6))
    {
        ModbusData.instant = ModbusData.oneImpulse * 3600000 / interval;
        prevSec = now;
        ModbusData.globalPump += ModbusData.oneImpulse;
    }
}
// все ок

void globalPumpTimer(void)
{
    ModbusData.pressureDD = GeneratorDD(ModbusData.DDLimit);
    if (GetDO(DO3))
        ModbusData.globalSeconds++;
    if (ModbusData.globalSeconds == 60)
    {
        ModbusData.globalMinutes++;
        ModbusData.globalSeconds = 0;
    }
    if (ModbusData.globalMinutes == 60)
    {
        ModbusData.globalHours++;
        ModbusData.globalMinutes = 0;
    }
    if (ModbusData.resetMotoHours)
    {
        ModbusData.globalHours = 0;
        ModbusData.globalMinutes = 0;
        ModbusData.globalSeconds = 0;
        ModbusData.resetMotoHours = 0;
    }
}
// все ок